@@ -1,3 +1,4 @@
+# -*- coding: utf-8 -*-
 import random
 import warnings
 import copy
@@ -25,25 +26,23 @@ except:
 
 PY2 = sys.version_info[0] == 2
 
-if not PY2:
-    long = int
+
+__version__ = '0.9.0'
 
 
-__version__ = '0.8.2'
-
-
-if sys.version_info[0] == 2:
+if PY2:
+    DEFAULT_ENCODING = 'utf-8'
     text_type = unicode
     string_types = (str, unicode)
     redis_string_types = (str, unicode, bytes)
     byte_to_int = ord
     int_to_byte = chr
 
-    def to_bytes(x, charset=sys.getdefaultencoding(), errors='strict'):
-        if isinstance(x, (bytes, bytearray, buffer)) or hasattr(x, '__str__'):
-            return bytes(x)
+    def to_bytes(x, charset=DEFAULT_ENCODING, errors='strict'):
         if isinstance(x, unicode):
             return x.encode(charset, errors)
+        if isinstance(x, (bytes, bytearray, buffer)) or hasattr(x, '__str__'):
+            return bytes(x)
         if hasattr(x, '__unicode__'):
             return unicode(x).encode(charset, errors)
         raise TypeError('expected bytes or unicode, not ' + type(x).__name__)
@@ -58,6 +57,9 @@ if sys.version_info[0] == 2:
     iteritems = lambda d: d.iteritems()
     from urlparse import urlparse
 else:
+    DEFAULT_ENCODING = sys.getdefaultencoding()
+    long = int
+    basestring = str
     text_type = str
     string_types = (str,)
     redis_string_types = (bytes, str)
@@ -130,6 +132,12 @@ class _StrKeyDict(MutableMapping):
     def expire(self, key, timestamp):
         self._ex_keys[key] = timestamp
 
+    def persist(self, key):
+        try:
+            del self._ex_keys[key]
+        except KeyError:
+            pass
+
     def expiring(self, key):
         if key not in self._ex_keys:
             return None
@@ -174,8 +182,10 @@ def DecodeGenerator(gen):
 
 
 def _decode(value):
-    if isinstance(value, bytes):
-        value = value.decode()
+    if isinstance(value, text_type):
+        return value
+    elif isinstance(value, bytes):
+        value = value.decode(DEFAULT_ENCODING)
     elif isinstance(value, dict):
         value = dict((_decode(k), _decode(v)) for k, v in value.items())
     elif isinstance(value, (list, set, tuple)):
@@ -403,7 +413,7 @@ class FakeStrictRedis(object):
         pass
 
     def persist(self, name):
-        pass
+        self._db.persist(name)
 
     def ping(self):
         return True
@@ -446,6 +456,8 @@ class FakeStrictRedis(object):
                 if px > 0:
                     self._db.expire(name, datetime.now() +
                                     timedelta(milliseconds=px))
+            else:
+                self._db.persist(name)
             self._db[name] = to_bytes(value)
             return True
         else:
@@ -474,6 +486,9 @@ class FakeStrictRedis(object):
     def setex(self, name, time, value):
         if isinstance(time, timedelta):
             time = int(timedelta_total_seconds(time))
+        if not isinstance(time, int):
+            raise ResponseError(
+                'value is not an integer or out of range')
         return self.set(name, value, ex=time)
 
     def psetex(self, name, time_ms, value):
@@ -491,7 +506,12 @@ class FakeStrictRedis(object):
         return result
 
     def setrange(self, name, offset, value):
-        pass
+        val = self._db.get(name, b"")
+        if len(val) < offset:
+            val += b'\x00' * (offset - len(val))
+        val = val[0:offset] + to_bytes(value) + val[offset+len(value):]
+        self.set(name, val)
+        return len(val)
 
     def strlen(self, name):
         try:
@@ -544,6 +564,9 @@ class FakeStrictRedis(object):
             return b'list'
         elif isinstance(key, set):
             return b'set'
+        else:
+            assert key is None
+            return b'none'
 
     def watch(self, *names):
         pass
@@ -963,12 +986,34 @@ class FakeStrictRedis(object):
         except KeyError:
             return None
 
-    def srandmember(self, name):
-        "Return a random member of set ``name``"
+    def srandmember(self, name, number=None):
+        """
+        If ``number`` is None, returns a random member of set ``name``.
+
+        If ``number`` is supplied, returns a list of ``number`` random
+        memebers of set ``name``.
+        """
         members = self._db.get(name, set())
-        if members:
+        if not members:
+            if number is not None:
+                return []
+            else:
+                return None
+
+        if number is None:
             index = random.randint(0, len(members) - 1)
             return list(members)[index]
+        elif len(members) <= number:
+            # We return them all, shuffled.
+            res = list(members)
+            random.shuffle(res)
+            return res
+        else:
+            member_list = list(members)
+            return [
+                member_list[i] for i
+                in sorted(random.sample(range(len(members)), number))
+            ]
 
     def srem(self, name, *values):
         "Remove ``value`` from set ``name``"
@@ -1450,11 +1495,15 @@ class FakeStrictRedis(object):
         self._db[dest] = new_zset
 
     def _list_or_args(self, keys, args):
-        # Based off of list_or_args from redis-py.
         # Returns a single list combining keys and args.
-        # A string can be iterated, but indicates
-        # keys wasn't passed as a list.
-        if isinstance(keys, string_types):
+        # Copy of list_or_args from redis-py.
+        try:
+            iter(keys)
+            # a string or bytes instance can be iterated, but indicates
+            # keys wasn't passed as a list
+            if isinstance(keys, (basestring, bytes)):
+                keys = [keys]
+        except TypeError:
             keys = [keys]
         if args:
             keys.extend(args)
@@ -1727,6 +1776,7 @@ class FakePubSub(object):
     SUBSCRIBE_MESSAGE_TYPES = ['subscribe', 'psubscribe']
     UNSUBSCRIBE_MESSAGE_TYPES = ['unsubscribe', 'punsubscribe']
     PATTERN_MESSAGE_TYPES = ['psubscribe', 'punsubscribe']
+    LISTEN_DELAY = 0.1          # delay between listen loops (seconds)
 
     def __init__(self, decode_responses=False, *args, **kwargs):
         self.channels = {}
@@ -1877,8 +1927,9 @@ class FakePubSub(object):
             message = self.get_message()
             if message:
                 yield message
+                continue
 
-            time.sleep(1)
+            time.sleep(self.LISTEN_DELAY)
 
     def close(self):
         """
